use Link;
use searchers::Searcher;

use scryfall;
use scryfall::{Scryfall, Card};

use retry;
use retry::retry;
use retry::delay::Fixed;

use std;

use mopa::Any;

const NUM_RETRIES: usize = 10;
const RETRY_WAIT_MILLIS: u64 = 500;

pub type MtgCard = Card;

impl Link for Card {
    fn label (&self) -> &str {
        &self.name
    }

    fn url (&self) -> &str {
        &self.uri
    }

    fn as_any (&self) -> &Any {
        self
    }
}

pub struct MtgSearcher {
    scryfall: Scryfall
}

impl MtgSearcher {
    pub fn new () -> MtgSearcher {
        MtgSearcher {
            scryfall: Scryfall::new()
        }
    }

    fn do_search (&self, query: &str) -> Result<Option<Card>, Error> {
        let mut result = retry(Fixed::from_millis(RETRY_WAIT_MILLIS).take(NUM_RETRIES), || {
            self.scryfall.search(query, None, 0)
        })?;

        Ok(if !result.data.is_empty() {
            Some(result.data.remove(0))
        } else {
            None
        })
    }
}

impl Searcher<Card> for MtgSearcher {
    fn exact_search (&self, name: &str) -> Option<Card> {
        let search = format!(r#"!"{}""#, name);
        self.do_search(&search).unwrap_or(None)
    }
}

#[derive(Debug)]
enum Error {
    Client(scryfall::Error),
    Io(std::io::Error),
}

impl From<scryfall::Error> for Error {
    fn from (error: scryfall::Error) -> Error {
        Error::Client(error)
    }
}

impl From<std::io::Error> for Error {
    fn from (error: std::io::Error) -> Error {
        Error::Io(error)
    }
}


impl From<retry::Error<scryfall::Error>> for Error {
    fn from (err: retry::Error<scryfall::Error>) -> Error {
        match err {
            retry::Error::Operation { error, total_delay, tries } => {
                Error::Client(error)
            }
        }
    }
}
